// CCar Class Implementation
// 
// Description : base car class, handles everything car-like
//

#include "stdafx.h"
#include "Car.h"
#include "Environment.h"

///////////////////////////////////////////////////////////////////
// CCar::CCar()
// Description:	constructor
///////////////////////////////////////////////////////////////////
CCar::CCar()
{	
	InitializeParameters();
	m_fMaxSpeed = 8.0f;
	m_fMinSpeed = -1.5f;
}

///////////////////////////////////////////////////////////////////
// CCar::InitializeParameters()
// Description:	inits variables to reset game
///////////////////////////////////////////////////////////////////
void CCar::InitializeParameters()
{
	m_fRotation			= 0.0f;
	m_fSpeed			= 0.0f;
	m_nQuadrant			= 5;
	m_nLap				= 0;
	m_bLap				= true;
	m_bFinished			= false;
	m_bWonRace			= false;
	m_ptDirection.set(-1.0f, 0.0f, 0.0f);
	m_ptVelocityVector.set(0.0f, 0.0f, 0.0f);
}

///////////////////////////////////////////////////////////////////
// CCar::Update()
// Description:	generic update function, called every time step
///////////////////////////////////////////////////////////////////
void CCar::Update()
{ 
	SetQuadrant();
}

///////////////////////////////////////////////////////////////////
// CCar::Draw()
// Description:	draw the stored model (in the correct orientation)
///////////////////////////////////////////////////////////////////
void CCar::Draw()
{ 
	glEnable(GL_BLEND);
	glRotatef(90.0f, 0.0f, 0.0f, 1.0f);
	glTranslatef(0.0f, 0.0f, 2.5f);
	m_cCarModel.Draw();
	glDisable(GL_BLEND);
}

///////////////////////////////////////////////////////////////////
// CCar::TurnLeft()
// Description:	turn the car left, update direction vector
///////////////////////////////////////////////////////////////////
void CCar::TurnLeft()
{ 
	if(m_fSpeed > 0.1f || m_fSpeed < -1.0f)
	{
		m_fRotation -= 0.04f;
		m_fSpeed -= 0.01f;
	}
	if (m_fRotation < 0.0f)
		m_fRotation += 2.0f * (float)GU_PI;
		
	CMatrix matTurn;
	matTurn.LoadIdentity();
	matTurn.rotate((float)(-GURad2Deg(m_fRotation)), 'z');
	m_ptDirection = CPnt(-1.0f, 0.0f, 0.0f) * matTurn;

	m_bIsTurning = true;
}

///////////////////////////////////////////////////////////////////
// CCar::TurnRight()
// Description:	turn the car right, update direction vector
///////////////////////////////////////////////////////////////////
void CCar::TurnRight()
{ 
	if(m_fSpeed > 0.1f || m_fSpeed < -1.0f)
	{
		m_fRotation += 0.04f;
		m_fSpeed -= 0.01f;
	}
	if (m_fRotation > 2.0f * (float)GU_PI)
		m_fRotation -= 2.0f * (float)GU_PI;

	CMatrix matTurn;
	matTurn.LoadIdentity();
	matTurn.rotate((float)(-GURad2Deg(m_fRotation)), 'z');
	m_ptDirection = CPnt(-1.0f, 0.0f, 0.0f) * matTurn; 

	m_bIsTurning = true;
}

///////////////////////////////////////////////////////////////////
// CCar::SpeedUp()
// Description:	speed up the car
///////////////////////////////////////////////////////////////////
void CCar::SpeedUp()
{
	m_fSpeed += 0.03f; 
	if (m_fSpeed > m_fMaxSpeed)
		m_fSpeed = m_fMaxSpeed;

	if(m_fSpeed > 0.0f)
		m_ptVelocityVector = (m_ptVelocityVector * 5.0f + (m_ptDirection * m_fMaxSpeed * (1.0f - (float)pow(10, -(m_fSpeed / m_fMaxSpeed))))) / 6.0f;
	else
		m_ptVelocityVector = m_ptDirection * m_fSpeed;
}

///////////////////////////////////////////////////////////////////
// CCar::Brake()
// Description:	slow down the car, continue to going in reverse
///////////////////////////////////////////////////////////////////
void CCar::Brake()
{
	if(m_fSpeed > 0.0f)
		m_fSpeed -= 0.25f;
	else
		m_fSpeed -= 0.1f;

	if (m_fSpeed < m_fMinSpeed)
		m_fSpeed = m_fMinSpeed;

	if(m_fSpeed > 0.0f)
		m_ptVelocityVector = (m_ptVelocityVector * 5.0f + (m_ptDirection * m_fMaxSpeed * (1.0f - (float)pow(10, -(m_fSpeed / m_fMaxSpeed))))) / 6.0f;
	else if(m_fSpeed < -1.0)
		m_ptVelocityVector = m_ptDirection * m_fSpeed;
}

///////////////////////////////////////////////////////////////////
// CCar::Coast()
// Description:	coast the car
///////////////////////////////////////////////////////////////////
void CCar::Coast()
{
	m_fSpeed -= 0.025f * m_fSpeed;

	if(m_fSpeed > 0.0f)
		m_ptVelocityVector = (m_ptVelocityVector * 5.0f + (m_ptDirection * m_fMaxSpeed * (1.0f - (float)pow(10, -(m_fSpeed / m_fMaxSpeed))))) / 6.0f;
	else
		m_ptVelocityVector = m_ptDirection * m_fSpeed;
}

///////////////////////////////////////////////////////////////////
// CCar::Move(int* pEnvironment)
// Description:	move the car (after updating speed, etc),
//				also checks to make sure it CAN move, and acts
//				accordingly
///////////////////////////////////////////////////////////////////
bool CCar::Move(int* pEnvironment)
{ 
	bool bCrash = false;
	CEnvironment* pEnv = (CEnvironment*) pEnvironment;
	CPnt ptTest = m_ptPosition + m_ptVelocityVector;

	float fDist = GU_MAX;
	CCar* pClosest = NULL;

	// see if this car is hitting the user car
	if(((CCar)pEnv->m_cUserCar).m_ptPosition != m_ptPosition && pEnv->m_cUserCar.CheckColliding(this) != NONE)
	{
		fDist = GUDistance(m_ptPosition, ((CCar)pEnv->m_cUserCar).m_ptPosition);
		pClosest = &((CCar)pEnv->m_cUserCar);
		bCrash = true;
	}

	// see if this car is hitting any enemy car
	if(!pClosest)
		for(int i = 0; i < NUM_ENEMY_CARS; i++)
		{
			if(((CCar)pEnv->m_acEnemyCars[i]).m_ptPosition != m_ptPosition && pEnv->m_acEnemyCars[i].CheckColliding(this) != NONE)			
				if(GUDistance(m_ptPosition, ((CCar)pEnv->m_acEnemyCars[i]).m_ptPosition) < fDist)
				{
					fDist = GUDistance(m_ptPosition, ((CCar)pEnv->m_acEnemyCars[i]).m_ptPosition);
					pClosest = &((CCar)(pEnv->m_acEnemyCars[i]));
					bCrash = true;
				}
		}

	// if it is hitting something, deal with the collision
	if(pClosest)
	{
		if(pClosest->CheckColliding(this) == FRONT)
		{
			m_fSpeed = m_fSpeed * 0.5f;
			pClosest->m_fSpeed = pClosest->m_fSpeed * 0.5f;
			m_ptVelocityVector = -m_ptVelocityVector;
			pClosest->m_ptVelocityVector = -pClosest->m_ptVelocityVector + m_ptVelocityVector * 0.5f;
		}
		else
		{
			m_fSpeed = m_fSpeed * 0.3f;
			pClosest->m_fSpeed = pClosest->m_fSpeed * 0.9f;
			pClosest->m_ptVelocityVector = pClosest->m_ptVelocityVector + m_ptVelocityVector * 0.5f;
			m_ptVelocityVector = -m_ptVelocityVector;
		}

		ptTest = m_ptPosition - m_ptDirection;
		pClosest->m_ptPosition = pClosest->m_ptPosition - pClosest->m_ptDirection;
	}

	// see if car hit a wall
	bool bHitWall = false;
	switch(pEnv->m_cTrack.CheckColliding(ptTest))
	{
	case RIGHT:
		m_fSpeed = m_fSpeed * 0.8f;
		TurnLeft(); TurnLeft(); TurnLeft();
		if(m_nQuadrant == 1 || m_nQuadrant == 4)
			m_ptVelocityVector = -m_ptVelocityVector * 0.75f;
		else
			m_ptVelocityVector[1] = -m_ptVelocityVector[1] * 0.75f;
		bHitWall = true;
		break;
	case LEFT:
		m_fSpeed = m_fSpeed * 0.8f;
		TurnRight(); TurnRight(); TurnRight();
		if(m_nQuadrant == 1 || m_nQuadrant == 4)
			m_ptVelocityVector = -m_ptVelocityVector * 0.75f;
		else
			m_ptVelocityVector[1] = -m_ptVelocityVector[1] * 0.75f;
		bHitWall = true;
		break;
	case NONE:
	default:
		bHitWall = false;
		break;
	}

	// if it hit a wall, deal with the collision
	if(bHitWall)
	{
		bCrash = true;
		for(int i = 100; i > 1; i-=10)
		{
			ptTest = m_ptPosition + m_ptVelocityVector * (float)i / 100.0f;
			if(pEnv->m_cTrack.CheckColliding(ptTest) == NONE)
				break;
		}
		if(i < 1)
			return bCrash;
	}
	m_ptPosition = ptTest;
	return bCrash;
}


///////////////////////////////////////////////////////////////////
// CCar::CheckColliding(CCar* pCar)
// Description:	check to see if the passed car is colliding with this car
///////////////////////////////////////////////////////////////////
int CCar::CheckColliding(CCar* pCar)
{
	CPnt ptCheck = pCar->m_ptPosition + pCar->m_ptDirection * 3.0f;
	if(GUDistance(ptCheck, (m_ptPosition - m_ptDirection * 3.0f)) < 6.0f)
		return BACK;
	else if(GUDistance(ptCheck, (m_ptPosition + m_ptDirection * 3.0f)) < 6.0f)
		return FRONT;
	return NONE;
}

///////////////////////////////////////////////////////////////////
// CCar::CheckCollidingPt(CPnt ptCar)
// Description:	check to see if the passed point is colliding with this car
///////////////////////////////////////////////////////////////////
int CCar::CheckCollidingPt(CPnt ptCar)
{
	if(GUDistance(ptCar, m_ptPosition) < 7.0f)
		return BACK;
	return NONE;
}

///////////////////////////////////////////////////////////////////
// CCar::SetQuadrant()
// Description:	figure out what place this car is in, and set member variable
///////////////////////////////////////////////////////////////////	
void CCar::SetQuadrant()
{
	if(GetPos().data[0] < 0.0f && GetPos().data[0] > -660.0f && GetPos().data[1] > 0.0f )
	{
		if(m_nQuadrant == 5)
		{
			m_nQuadrant = 0;
			if(m_bLap)
			{
				m_nLap++;
			}
			m_bLap = false;
		}
	}
	else if(GetPos().data[0] < 0.0f && GetPos().data[0] > -660.0f && GetPos().data[1] < 0.0f )
	{
		if(m_nQuadrant == 1)
			m_nQuadrant = 2;
	}
	else if(GetPos().data[0] > 0.0f && GetPos().data[0] < 660.0f && GetPos().data[1] < 0.0f )
	{
		if(m_nQuadrant == 2)
			m_nQuadrant = 3;
	}
	else if(GetPos().data[0] > 0.0f && GetPos().data[0] < 660.0f && GetPos().data[1] > 0.0f )
	{
		if(m_nQuadrant == 4)
		{
			m_bLap = true;
			m_nQuadrant = 5;
		}
	}
	else if(GetPos().data[0] > 660.0f)
	{
		if(m_nQuadrant == 3)
			m_nQuadrant = 4;
	}
	else if(GetPos().data[0] < -660.0f)
	{
		if(m_nQuadrant == 0)
		{
			m_nQuadrant = 1;
		}
	}
}


///////////////////////////////////////////////////////////////////
// CCar::GetPlaceString()
// Description:	format place as string
///////////////////////////////////////////////////////////////////	
string CCar::GetPlaceString()
{
	char buffer[10];
	sprintf(buffer, "%.2d", m_nPlace);
	return buffer;
} 

///////////////////////////////////////////////////////////////////
// CCar::GetLapString()
// Description:	format laps as string
///////////////////////////////////////////////////////////////////	
string CCar::GetLapString()
{
	char buffer[10];
	sprintf(buffer, "%.2d", m_nLap);
	return buffer;
} 

///////////////////////////////////////////////////////////////////
// CCar::GetNumLapsString()
// Description:	format number of laps as string
///////////////////////////////////////////////////////////////////	
string CCar::GetNumLapsString()
{
	char buffer[10];
	sprintf(buffer, "%.2d", m_nNumLaps);
	return buffer;
} 