// CGameEngine Class Implementation
// 
// Description:		top level class of game.
//					handles messages, input, game states, sound, music, and
//					basic program management
//

// includes
#include "stdafx.h"
#include "Images_v1.3.h"
#include "halcyon.h"
#include "GameEngine.h"
#include "UserCar.h"
#include "Car.h"
//#include "winuser.h"
#include <iostream>
#include <fstream>
using namespace std;

// game state defines
#define GS_LOADINGSCREEN	0
#define GS_ABOUTSCREEN		1
#define GS_MAINSCREEN		2
#define GS_HIGHSCORE		3
#define GS_GAMESTART		4
#define GS_GAME				5
#define GS_GAMECLOSE		6
#define GS_NEWHIGHSCORE		7
#define	GS_GAMEOVER			8
#define NUM_GS				9

//////////////////////////////////////////////////////////////////////////////////// message mapping

BEGIN_MESSAGE_MAP(CGameEngine, COpenGLControlBase)
	//{{AFX_MSG_MAP(CGameEngine)
	ON_WM_TIMER()
	ON_WM_KEYDOWN()
	ON_WM_KEYUP()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

//////////////////////////////////////////////////////////////////////////////////// basic stuff, updates

///////////////////////////////////////////////////////////////////
// CGameEngine::CGameEngine()
// Description:	constructor
///////////////////////////////////////////////////////////////////
CGameEngine::CGameEngine()
{
	m_nMenuItemSelected = 0;
	m_nHighScorePlace = 0;
	m_bUp = false;
	m_bDown = false;
	m_bLeft = false;
	m_bRight = false;
	m_nFadeType = FADE_NONE;
	m_fFade = 0.0f;

	LoadHighScores();

	m_cResChanger.ChangeResolution(640, 480, 32);
}

///////////////////////////////////////////////////////////////////
// CGameEngine::~CGameEngine()
// Description:	destructor
///////////////////////////////////////////////////////////////////
CGameEngine::~CGameEngine()
{
	m_cResChanger.RestoreResoltion();
	SaveHighScores();
}

///////////////////////////////////////////////////////////////////
// CGameEngine::vWindowInitialExposure()
// Description:	called the first time the window is displayed
//				sets up initial lights, blending, etc
///////////////////////////////////////////////////////////////////
void CGameEngine::vWindowInitialExposure()
{
	glClearColor(0.0f, 0.0f, 0.0f, 1.0f);

	glViewport (0, 0, 640, 480);
	glMatrixMode (GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(50.0f, 1.333f, 1.0f, 100.0f);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	gluLookAt(0.0f, 0.0f, -5.0f, 0.0f, 0.0f, 5.0f, 0.0f, 1.0f, 0.0f);

	//lighting
	GLfloat lightpos[4] = { 0.0f, 3.0f, -5.0f, 0.0f };
	GLfloat lightamb[4] = { 0.3f, 0.3f, 0.3f, 1.0f };
	GLfloat lightdif[4] = { 1.0f, 1.0f, 1.0f, 1.0f };
	GLfloat lightspe[4] = { 1.0f, 1.0f, 1.0f, 1.0f };

	glLightfv(GL_LIGHT0, GL_POSITION, lightpos);
	glLightfv(GL_LIGHT0, GL_AMBIENT, lightamb);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, lightdif);
	glLightfv(GL_LIGHT0, GL_SPECULAR, lightspe);
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);

	GLfloat lmodel_ambient[] = { 0.3f, 0.3f, 0.3f, 1.0f };
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, lmodel_ambient);

	//texture + blending
	glDisable(GL_BLEND);
	glEnable(GL_TEXTURE_2D);
	glShadeModel(GL_SMOOTH);
	glEnable(GL_DEPTH_TEST);
	glDisable(GL_FOG);

	m_nGameState = GS_LOADINGSCREEN;
	SetTimer(UPDATE_TIMER, 1, NULL);	
}

///////////////////////////////////////////////////////////////////
// CGameEngine::OnTimer(UINT nIDEvent) 
// Description:	timer messages get sent here
///////////////////////////////////////////////////////////////////
void CGameEngine::OnTimer(UINT nIDEvent) 
{	
	if (nIDEvent == UPDATE_TIMER)
	{
		vWindowUpdate();
		Invalidate();
	}
	COpenGLControlBase::OnTimer(nIDEvent);
}

///////////////////////////////////////////////////////////////////
// CGameEngine::vWindowUpdate()
// Description:	general update function
///////////////////////////////////////////////////////////////////
void CGameEngine::vWindowUpdate()
{
	// if it should be fading, update fading values
	if(m_nFadeType == FADE_IN)
	{
		m_fFade -= 0.03f;
		if(m_fFade < 0.0f)
		{
			m_fFade = 0.0f;
			m_nFadeType = FADE_NONE;
		}
	}
	else if(m_nFadeType == FADE_OUT)
	{
		m_fFade += 0.03f;
		if(m_fFade > 1.0f)
		{
			m_fFade = 1.0f;
			m_nFadeType = FADE_NONE;
			switch(m_nNextGameState)
			{
			case GS_LOADINGSCREEN:
				break;
			case GS_ABOUTSCREEN:
				m_nGameState = GS_ABOUTSCREEN;
				m_fFade = 1.0f;
				m_nFadeType = FADE_IN;	
				break;
			case GS_MAINSCREEN:
				InitMainScreen();
				break;
			case GS_HIGHSCORE:
				InitHighScore();
				break;
			case GS_GAMESTART:
				InitGameStart();
				break;
			case GS_GAME:
				InitGamePlay();
				break;
			case GS_GAMECLOSE:
				InitGameClose();
				break;
			case GS_NEWHIGHSCORE:
				InitHighScore();
				break;
			case GS_GAMEOVER:
				break;
			default:
				break;
			}
		}
	}
	
	// updates are different for each gamestate
	switch(m_nGameState)
	{
	case GS_LOADINGSCREEN:  // loading screen only gets updated once, at the beginning
							// it needs to set up everything, display lists, models, sound, etc
		{
			// set up text drawing display lists
			LOGFONT     lf;
			HFONT       hFont;
			GLYPHMETRICSFLOAT agmf[128];

			wglMakeCurrent(wglGetCurrentDC(), m_hRC );

			memset(&lf,0,sizeof(LOGFONT));
			lf.lfHeight               =   -20;
			lf.lfWeight               =   FW_BOLD;
			lf.lfCharSet              =   ANSI_CHARSET;
			lf.lfOutPrecision         =   OUT_DEFAULT_PRECIS;
			lf.lfClipPrecision        =   CLIP_DEFAULT_PRECIS;
			lf.lfQuality              =   DEFAULT_QUALITY;
			lf.lfPitchAndFamily       =   FF_DONTCARE|DEFAULT_PITCH;
			lstrcpy (lf.lfFaceName, "Arial") ;

			hFont = CreateFontIndirect(&lf);
			SelectObject(wglGetCurrentDC(),hFont);	

			m_dlFontLists = glGenLists(128);

			if (!(wglUseFontOutlines(wglGetCurrentDC(), 0, 128, m_dlFontLists, 0.0f, 0.05f, WGL_FONT_POLYGONS, agmf)))
				MessageBox("wglUseFontOutlines failed!","GLFont",MB_OK);

			// send text dl index to everything that needs it
			m_cEnv.m_cUserCar.m_dlText = m_dlFontLists;
			for(int i = 0; i < NUM_ENEMY_CARS; i++)
				m_cEnv.m_acEnemyCars[i].m_dlText = m_dlFontLists;

			// draw the loading screen
			glTranslatef(0.7f, 0.0f, 0.0f);
			glRotatef(180.0f, 0.0f, 1.0f, 0.0f);
			glScalef(0.25f, 0.25f, 0.25f);
			glClear(GL_COLOR_BUFFER_BIT);
			DrawStr("Loading...");
			glFinish();
			SwapBuffers(m_pDC->GetSafeHdc()); // force the buffers to swap to display the message

			// load all the models needed
			m_cEnv.LoadModels();
			m_cMetalPlate.Load("Models/metalplate.iv");
			m_cStartScreenCar.Load("Models/porsche.o.iv");
			m_cPlatform.Load("Models/platform.iv");

			// set up color display lists
			GLfloat selectedmat[] = { 1.0f, 0.0f, 0.0f, 1.0f };
			m_dlRedMaterial = glGenLists(1);
			glNewList(m_dlRedMaterial, GL_COMPILE);
				glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, selectedmat);
			glEndList();

			GLfloat unselectedmat[] = { 0.0f, 0.4f, 0.9f, 1.0f };
			m_dlBlueMaterial = glGenLists(1);
			glNewList(m_dlBlueMaterial, GL_COMPILE);
				glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, unselectedmat);
			glEndList();

			// set up background display list
			string strBackground = "Models/background.rgb";
			COpenGLImage cBack(strBackground);
			GLuint texBack = cBack.MakeTexture();
			
			m_dlBack = glGenLists(1);
			glNewList(m_dlBack, GL_COMPILE_AND_EXECUTE);
			GLfloat backmat[] = { 0.8f, 0.8f, 0.8f, 1.0f };
			glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, backmat);
			glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
			glBindTexture(GL_TEXTURE_2D, texBack);
			glNormal3f(0.0f, 0.0f, 1.0f);
			glBegin(GL_POLYGON);
				glTexCoord2f(0.0f, 0.0f);
				glVertex3f(-26.0f, -20.0f, 35.0f);
				glTexCoord2f(0.0f, 1.0f);
				glVertex3f(-26.0f, 20.0f, 35.0f);
				glTexCoord2f(1.0f, 1.0f);
				glVertex3f(26.0f, 20.0f, 35.0f);
				glTexCoord2f(1.0f, 0.0f);
				glVertex3f(26.0f, -20.0f, 35.0f);
			glEnd( );
			glEndList();

			// set up sound
			if(!m_cSound.Init())
				AfxMessageBox("Doh! Sound didn't Init");
			m_cSound.AddNewBuffer("Sounds/Repeat.wav");
			m_cSound.AddNewBuffer("Sounds/block.wav");
			m_cSound.AddNewBuffer("Sounds/goodswoosh.wav");
			m_cSound.AddNewBuffer("Sounds/intro.wav");
			m_cSound.AddNewBuffer("Sounds/1.wav");
			m_cSound.AddNewBuffer("Sounds/2.wav");
			m_cSound.AddNewBuffer("Sounds/3.wav");
			m_cSound.AddNewBuffer("Sounds/4.wav");
			m_cSound.AddNewBuffer("Sounds/5.wav");
			m_cSound.AddNewBuffer("Sounds/crash.wav");
			m_cSound.AddNewBuffer("Sounds/gamemusic.wav");
			m_cSound.AddNewBuffer("Sounds/Skidmid.wav");
			m_cSound.AddNewBuffer("Sounds/goodswoosh2.wav");

			// go to mainscreen next
			InitMainScreen();
		}
		break;
	case GS_ABOUTSCREEN:
		break;
	case GS_MAINSCREEN:
		// rotate the car
		m_fRotateStartScreenCar+=1.25f;
		if(m_fRotateStartScreenCar > 360.0f)
			m_fRotateStartScreenCar -= 360.0f;
		break;
	case GS_HIGHSCORE:
		// update high score screen environment
		m_cEnv.UpdateHighScoreScreen();
		break;
	case GS_GAMESTART:
		// update intro animation
		if(((float)(clock() - m_fIntroStartTime) / CLOCKS_PER_SEC) > 5.3f)
		{
			m_nGameState = GS_GAME;
			InitGamePlay();
			m_cEnv.m_cUserCar.ToggleInGame(true);
		}
		else
			m_cEnv.UpdateGameStart(); // ready to play, go to game gamestate
		break;
	case GS_GAME:
		// update everything in the game
		if(m_cEnv.m_cUserCar.GetFinished())
		{
			if(m_bJustFinished)
			{
				m_fOutroStartTime = clock();
				m_bJustFinished = false;
			}
		}
		if(m_cEnv.m_cUserCar.GetFinished() && (((float)(clock() - m_fOutroStartTime) / CLOCKS_PER_SEC) > 2.0f))
		{
			if(m_cEnv.m_cUserCar.WonRace())
			{
				m_cSound.PlayBuffer(12, 0);
				InitHighScore();
				m_nHighScorePlace = CheckHighScore(m_cEnv.GetWinningTime());
				if(m_nHighScorePlace < 7)
				{
					//	move other high scores
					for( int i = 6; i > m_nHighScorePlace; i--)
					{
						m_strHighScores[i][0] = m_strHighScores[i - 1][0];
						m_strHighScores[i][1] = m_strHighScores[i - 1][1];
					}
					m_strHighScores[m_nHighScorePlace][1] = m_cEnv.GetWinningTime();
					m_strHighScores[m_nHighScorePlace][0] = "???";
					m_nGameState = GS_NEWHIGHSCORE;
				}
				else
					m_nGameState = GS_HIGHSCORE;
			}
			else
			{
				InitHighScore();
				m_nGameState = GS_HIGHSCORE;
			}
		}
		else
		{
			float Speed;
			Speed = (m_cEnv.m_cUserCar.GetVel().magnitude() * 30.0f);

			// play skid
			if(m_cEnv.m_cUserCar.m_bIsTurning && Speed > 60.0f)
				m_cSound.PlayBuffer(11, DSBPLAY_LOOPING);
			else
				m_cSound.StopBuffer(11);

			// play engine noise
			if(Speed < 40.0f)
			{
				m_cSound.StopBuffer(5);
				m_cSound.StopBuffer(6);
				m_cSound.StopBuffer(7);
				m_cSound.StopBuffer(4);
				m_cSound.PlayBuffer(8, DSBPLAY_LOOPING);
			}
			else if(Speed < 80.0f)
			{
				m_cSound.StopBuffer(4);
				m_cSound.StopBuffer(6);
				m_cSound.StopBuffer(5);
				m_cSound.StopBuffer(8);
				m_cSound.PlayBuffer(7, DSBPLAY_LOOPING);
			}
			else if(Speed < 130.0f)
			{
				m_cSound.StopBuffer(5);
				m_cSound.StopBuffer(4);
				m_cSound.StopBuffer(7);
				m_cSound.StopBuffer(8);
				m_cSound.PlayBuffer(6, DSBPLAY_LOOPING);
			}
			else if(Speed < 175.0f)
			{
				m_cSound.StopBuffer(7);
				m_cSound.StopBuffer(6);
				m_cSound.StopBuffer(4);
				m_cSound.StopBuffer(8);
				m_cSound.PlayBuffer(5, DSBPLAY_LOOPING);
			}
			else
			{
				m_cSound.StopBuffer(5);
				m_cSound.StopBuffer(6);
				m_cSound.StopBuffer(7);
				m_cSound.StopBuffer(8);
				m_cSound.PlayBuffer(4, DSBPLAY_LOOPING);
			}

			// play crash sound if hitting something
			if(m_cEnv.Update(m_bUp, m_bDown, m_bLeft, m_bRight, m_chLastKeyHit))
			{
				m_cSound.PlayBuffer(9, 0);
			}

			// reset last key hit
			m_chLastKeyHit = NULL;
		}
		break;
	case GS_GAMECLOSE:
		PostQuitMessage(1); // quit the game
		break;
	case GS_NEWHIGHSCORE:
		m_cEnv.UpdateHighScoreScreen(); // update the high score entry screen
		break;
	default:
		break;
	}
}


//////////////////////////////////////////////////////////////////////////////////// Inits

///////////////////////////////////////////////////////////////////
// CGameEngine::InitMainScreen()
// Description:	initialize everything for the main screen
///////////////////////////////////////////////////////////////////
void CGameEngine::InitMainScreen()
{
	// change gamestate
	m_nGameState = GS_MAINSCREEN;
	
	// fading
	m_fFade = 1.0f;
	m_nFadeType = FADE_IN;

	// setup opengl viewport
	glClearColor(0.0f, 0.0f, 0.0f, 1.0f);
	glViewport (0, 0, 640, 480);
	glMatrixMode (GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(50.0f, 1.333f, 1.0f, 100.0f);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	gluLookAt(0.0f, 0.0f, -5.0f, 0.0f, 0.0f, 5.0f, 0.0f, 1.0f, 0.0f);
	
	// reset menu
	m_nMenuItemSelected = 0;

	//lighting
	GLfloat lightpos[4] = { 0.0f, 3.0f, -5.0f, 0.0f };
	GLfloat lightamb[4] = { 0.4f, 0.4f, 0.4f, 1.0f };
	GLfloat lightdif[4] = { 1.0f, 1.0f, 1.0f, 1.0f };
	GLfloat lightspe[4] = { 1.0f, 1.0f, 1.0f, 1.0f };

	glLightfv(GL_LIGHT0, GL_POSITION, lightpos);
	glLightfv(GL_LIGHT0, GL_AMBIENT, lightamb);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, lightdif);
	glLightfv(GL_LIGHT0, GL_SPECULAR, lightspe);
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);

	GLfloat lmodel_ambient[] = { 0.3f, 0.3f, 0.3f, 1.0f };
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, lmodel_ambient);

	//texture + blending
	glDisable(GL_BLEND);
	glEnable(GL_TEXTURE_2D);
	glShadeModel(GL_SMOOTH);
	glEnable(GL_DEPTH_TEST);
	glDisable(GL_FOG);

	// music
	m_cSound.PlayBuffer(0, DSBPLAY_LOOPING);
}

///////////////////////////////////////////////////////////////////
// CGameEngine::InitGameStart()
// Description:	sets up game viewing environment and intro animation
///////////////////////////////////////////////////////////////////
void CGameEngine::InitGameStart()
{
	// sound
	m_cSound.StopAllBuffers();
	m_cSound.PlayBuffer(10, DSBPLAY_LOOPING);

	// fading
	m_fFade = 1.0f;
	m_nFadeType = FADE_IN;
	
	// gamestate stuff
	m_bJustFinished = true;
	m_nGameState = GS_GAMESTART;

	// opengl viewport
	glClearColor(0.0f, 0.0f, 0.0f, 1.0f);
	glViewport (0, 0, 640, 480);
	glMatrixMode (GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(57.0f, 1.333f, 1.0f, 3000.0f);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	gluLookAt(0.0f, 0.0f, 5.0f, 0.0f, 0.0f, -100.0f, 0.0f, 1.0f, 0.0f);

	// lighting
	glEnable(GL_LIGHTING);
	GLfloat lightpos[4] = { 0.0f, 100.0f, 0.0f, 0.0f };
	GLfloat lightamb[4] = { 0.3f, 0.3f, 0.3f, 1.0f };
	GLfloat lightdif[4] = { 0.8f, 0.8f, 0.8f, 1.0f };

	glLightfv(GL_LIGHT0, GL_POSITION, lightpos);
	glLightfv(GL_LIGHT0, GL_AMBIENT, lightamb);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, lightdif);
	glEnable(GL_LIGHT0);

	// init environment
	m_cEnv.InitializeEnvironment();
}

///////////////////////////////////////////////////////////////////
// CGameEngine::InitGamePlay()
// Description:	inits the game play after the intro
///////////////////////////////////////////////////////////////////
void CGameEngine::InitGamePlay()
{	
	// key flags reset
	m_bUp = false;
	m_bDown = false;
	m_bLeft = false;
	m_bRight = false;

	m_nGameState = GS_GAME;
}

///////////////////////////////////////////////////////////////////
// CGameEngine::InitGameClose()
// Description:	initialize game outro (nothing here now really)
///////////////////////////////////////////////////////////////////
void CGameEngine::InitGameClose()
{
	// opengl viewport
	glMatrixMode (GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(50.0f, 1.333f, 1.0f, 100.0f);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	gluLookAt(0.0f, 0.0f, 5.0f, 0.0f, 0.0f, -100.0f, 0.0f, 1.0f, 0.0f);

	m_nGameState = GS_GAMECLOSE;
}

///////////////////////////////////////////////////////////////////
// CGameEngine::InitHighScore()
// Description:	inits the high score screen
///////////////////////////////////////////////////////////////////
void CGameEngine::InitHighScore()
{
	// fading
	m_fFade = 1.0f;
	m_nFadeType = FADE_IN;

	// stops engine sounds
	m_cSound.StopBuffer(4);
	m_cSound.StopBuffer(5);
	m_cSound.StopBuffer(6);
	m_cSound.StopBuffer(7);
	m_cSound.StopBuffer(8);
	m_cSound.StopBuffer(10);

	// gamestate change
	m_cEnv.m_cUserCar.ToggleInGame(false);
	m_nGameState = GS_HIGHSCORE;

	// position for entering new initials
	m_nHighScoreInitialsPosition = 0;

	// opengl viewport
	glClearColor(0.0f, 0.0f, 0.0f, 1.0f);
	glViewport (0, 0, 640, 480);
	glMatrixMode (GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(57.0f, 1.333f, 1.0f, 3000.0f);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	gluLookAt(0.0f, 0.0f, 5.0f, 0.0f, 0.0f, -100.0f, 0.0f, 1.0f, 0.0f);

	// lighting
	glEnable(GL_LIGHTING);
	GLfloat lightpos[4] = { 0.0f, 100.0f, 0.0f, 0.0f };
	GLfloat lightamb[4] = { 0.9f, 1.0f, 0.9f, 1.0f };
	GLfloat lightdif[4] = { 0.7f, 0.7f, 0.7f, 1.0f };

	glLightfv(GL_LIGHT0, GL_POSITION, lightpos);
	glLightfv(GL_LIGHT0, GL_AMBIENT, lightamb);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, lightdif);
	glEnable(GL_LIGHT0);

	// environment init
	m_cEnv.InitializeHighScoreScreen();
}

//////////////////////////////////////////////////////////////////////////////////// drawing

///////////////////////////////////////////////////////////////////
// CGameEngine::vWindowDraw()
// Description:	handles drawing for each gamestate
///////////////////////////////////////////////////////////////////
void CGameEngine::vWindowDraw()
{
	// clear the screen
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glFlush();
	glPushMatrix();
		// call draw for the corresponding state
		switch (m_nGameState)
		{
		case GS_LOADINGSCREEN:
			break;
		case GS_HIGHSCORE:	 
			DrawHighScores();
			break;
		case GS_ABOUTSCREEN:
			DrawAboutScreen();			
			break;
		case GS_MAINSCREEN:
			DrawMainScreen();
			break;
		case GS_GAMESTART:
			m_cEnv.Draw();
			break;
		case GS_GAME:
			m_cEnv.Draw();
			break;
		case GS_GAMECLOSE:			
			PostQuitMessage(1);
			break;
		case GS_GAMEOVER:
			break;
		case GS_NEWHIGHSCORE:
			DrawNewHighScores();
			break;
		default:
			break;
		}
	glPopMatrix();

	// draw fading if necessary
	if(m_nGameState != GS_LOADINGSCREEN)
	{
		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity( );
		gluLookAt(0.0f, 0.0f, -5.0f, 0.0f, 0.0f, 5.0f, 0.0f, 1.0f, 0.0f);
		
		glDisable(GL_LIGHTING);
		glEnable(GL_BLEND);
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		glColor4f(0.0f, 0.0f, 0.0f, m_fFade);
		glBegin(GL_POLYGON);
			glVertex3f(-20.0f, -20.0f, -2.0f);
			glVertex3f(-20.0f, 20.0f, -2.0f);
			glVertex3f(20.0f, 20.0f, -2.0f);
			glVertex3f(20.0f, -20.0f, -2.0f);
		glEnd();
		glDisable(GL_BLEND);
		glEnable(GL_LIGHTING);

		if(m_nGameState == GS_GAMESTART)
			m_cEnv.UpdateGameStartCamera();
		else if(m_nGameState == GS_HIGHSCORE || m_nGameState == GS_NEWHIGHSCORE)
			m_cEnv.UpdateHighScoreCamera();
	}
}

///////////////////////////////////////////////////////////////////
// CGameEngine::DrawAboutScreen()
// Description:	draw the about screen
///////////////////////////////////////////////////////////////////
void CGameEngine::DrawAboutScreen()
{
	glPushMatrix();
		glCallList(m_dlBack);
		glTranslatef(-20.0f, 0.0f, 0.0f);
		m_cMetalPlate.Draw();
		glTranslatef(20.0f, 0.0f, 0.0f);

		glRotatef(180.0f, 0.0f, 1.0f, 0.0f);
		glTranslatef(-2.75f, 1.0f, -1.0f);
		glScalef(0.65f, 0.8f, 0.8f);

		GLfloat titlemat[] = { 0.8f, 0.8f, 0.0f, 1.0f };
		glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, titlemat);
		glPushMatrix();
			DrawStr("= Halcyon Racer =");
		glPopMatrix();
		glTranslatef(-0.5f, -2.0f, 0.0f);
		glScalef(0.5f, 0.5f, 0.5f);
		titlemat[2] = 0.8f;
		glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, titlemat);
		glPushMatrix();
		DrawStr("Halcyon Development:");
		glPopMatrix();
		glTranslatef(0.0f, -1.5f, 0.0f);
		glPushMatrix();
		DrawStr("Greg Croft, Andy Finkbeiner, Brian Fitzgerald");
		glPopMatrix();
		glTranslatef(0.0f, -1.0f, 0.0f);
		DrawStr("Emily Williams, and Marty Morgan");
	glPopMatrix();
}


///////////////////////////////////////////////////////////////////
// CGameEngine::DrawMainScreen()
// Description:	main screen drawing function
///////////////////////////////////////////////////////////////////
void CGameEngine::DrawMainScreen()
{
	// Stop Engine Sounds
	m_cSound.StopBuffer(4);
	m_cSound.StopBuffer(5);
	m_cSound.StopBuffer(6);
	m_cSound.StopBuffer(7);
	m_cSound.StopBuffer(8);
	
	glPushMatrix();
		// draw background polygon
		glCallList(m_dlBack);

		// draw menu items
		glTranslatef(-8.0f, -4.5f, 14.0f);
		glRotatef(90.0f, 1.0f, 0.0f, 0.0f);
		m_cMetalPlate.Draw();
		glRotatef(-90.0f, 1.0f, 0.0f, 0.0f);

		// draw text
		glPushMatrix();
			glRotatef(180.0f, 0.0f, 1.0f, 0.0f);
			glTranslatef(-2.5f, 2.5f, 1.0f);
			glScalef(0.65f, 0.8f, 0.8f);

			glPushMatrix();
				if(m_nMenuItemSelected == 0)
					DrawStr("Start Game", m_dlRedMaterial);
				else
					DrawStr("Start Game", m_dlBlueMaterial);
			glPopMatrix();

			glTranslatef(0.0f, -1.5f, 0.0f);
			glPushMatrix();
				if(m_nMenuItemSelected == 1)
					DrawStr("Best Times", m_dlRedMaterial);
				else
					DrawStr("Best Times", m_dlBlueMaterial);
			glPopMatrix();

			glTranslatef(0.0f, -1.5f, 0.0f);
			glPushMatrix();
				if(m_nMenuItemSelected == 2)
					DrawStr("About Halcyon", m_dlRedMaterial);
				else
					DrawStr("About Halcyon", m_dlBlueMaterial);
			glPopMatrix();

			glTranslatef(0.0f, -3.5f, 0.0f);
			if(m_nMenuItemSelected == 3)
					DrawStr("Quit", m_dlRedMaterial);
				else
					DrawStr("Quit", m_dlBlueMaterial);

			glPushMatrix();
				glTranslatef(-26.5f, 12.5f, 0.0f);
				GLfloat titlemat[] = { 0.6f, 0.6f, 0.0f, 1.0f };
				glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, titlemat);
				glRotatef(15.0f, 2.0f, 0.0f, 1.0f);
				glScalef(8.0f, 7.0f, 20.0f);
				DrawStr("Halcyon");
			glPopMatrix();

			glTranslatef(-12.5f, 11.5f, 0.0f);
			GLfloat racermat[] = { 1.0f, 0.0f, 0.0f, 1.0f };
			glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, racermat);
			glScalef(6.0f, 2.0f, 7.0f);
			DrawStr("RACER");
		glPopMatrix();

		// draw spinning car
		glTranslatef(11.5f, 3.0f, 2.0f);
		glPushMatrix();
			glTranslatef(0.0f, 0.0f, 0.25f);
			glRotatef(-110.0f, 1.0f, 0.0f, -0.05f);
			glRotatef(m_fRotateStartScreenCar, 0.0f, 0.0f, -1.0f);
			glEnable(GL_BLEND);
			m_cStartScreenCar.Draw();
			glDisable(GL_BLEND);
		glPopMatrix();
		glTranslatef(0.0f, -1.85f, 0.0f);
		glRotatef(-110.0f, 1.0f, 0.0f, -0.05f);
		glRotatef(m_fRotateStartScreenCar, 0.0f, 0.0f, -1.0f);
		m_cPlatform.Draw();
	glPopMatrix();
}

///////////////////////////////////////////////////////////////////
// CGameEngine::DrawHighScores()
// Description:	draw the high score screen
///////////////////////////////////////////////////////////////////
void CGameEngine::DrawHighScores()
{
	// draw the track & cars
	m_cEnv.DrawHighScoreScreen();
	
	// draw the translucent board
	glDisable(GL_LIGHTING);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glColor4f(0.0f, 0.0f, 0.0f, 0.7f);
	glBegin(GL_POLYGON);
		glVertex3f(-5.0f, -303.0f, 3.0f);
		glVertex3f(-5.0f, -297.0f, 3.0f);
		glVertex3f(-5.0f, -297.0f, 7.0f);
		glVertex3f(-5.0f, -303.0f, 7.0f);
	glEnd();
	glDisable(GL_BLEND);
	glLineWidth(3.0f);
	glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
	glColor4f(0.7f, 0.7f, 0.0f, 1.0f);
	glBegin(GL_POLYGON);
		glVertex3f(-5.0f, -303.0f, 3.0f);
		glVertex3f(-5.0f, -297.0f, 3.0f);
		glVertex3f(-5.0f, -297.0f, 7.0f);
		glVertex3f(-5.0f, -303.0f, 7.0f);
	glEnd();
	glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
	glEnable(GL_LIGHTING);

	// draw high score text
	glTranslatef(-4.0f, -302.28f, 6.25f);
	glRotatef(90.0f, 0.0f, 0.0f, 1.0f);
	glRotatef(90.0f, 1.0f, 0.0f, 0.0f);
	glScalef(0.4f, 0.4f, 0.4f);
	GLfloat highdif[] = { 0.8f, 0.8f, 0.8f, 1.0f };
	GLfloat highamb[] = { 0.5f, 0.5f, 0.5f, 1.0f };
	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, highdif);
	glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, highamb);
	glPushMatrix();
	DrawStr("Halcyon Racer Best Times", m_dlBlueMaterial);
	glPopMatrix();
	DrawStr("_______________________", m_dlBlueMaterial);
	glTranslatef(-11.25f, -1.0f, 0.0f);
	
	for(int i = 0; i < 7; i++)
	{
		glPushMatrix();
			char buffer[10];
			sprintf(buffer, "%d ", i+1);
			DrawStr(buffer, m_dlBlueMaterial);
		glPopMatrix();
		glPushMatrix();
			glTranslatef(1.0f, 0.0f, 0.0f);
			glColor4f(0.7f, 0.7f, 0.0f, 1.0f);
			DrawStr(m_strHighScores[i][0], m_dlBlueMaterial);
		glPopMatrix();
		glPushMatrix();
			glTranslatef(7.0f, 0.0f, 0.0f);
			DrawStr(m_strHighScores[i][1], m_dlBlueMaterial);
		glPopMatrix();
		glTranslatef(0.0f, -1.0f, 0.0f);
	}

}

///////////////////////////////////////////////////////////////////
// CGameEngine::DrawStr(string strMessage, const GLuint dlMaterial)
// Description:	function to draw text strings as opengl objects
///////////////////////////////////////////////////////////////////
void CGameEngine::DrawStr(string strMessage, const GLuint dlMaterial)
{
	glPushAttrib(GL_POLYGON_BIT | GL_ENABLE_BIT);
		if(dlMaterial > 0)
			glCallList(dlMaterial);
		glListBase(m_dlFontLists);
		glCallLists(strMessage.size(), GL_UNSIGNED_BYTE, strMessage.c_str());
	glPopAttrib();
}

///////////////////////////////////////////////////////////////////
// CGameEngine::DrawNewHighScores()
// Description:	draw while entering a new high score
///////////////////////////////////////////////////////////////////
void CGameEngine::DrawNewHighScores()
{
	// draw cars & track
	m_cEnv.DrawHighScoreScreen();

	// draw board
	glDisable(GL_LIGHTING);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glColor4f(0.0f, 0.0f, 0.0f, 0.7f);
	glBegin(GL_POLYGON);
		glVertex3f(-5.0f, -303.0f, 3.0f);
		glVertex3f(-5.0f, -297.0f, 3.0f);
		glVertex3f(-5.0f, -297.0f, 7.0f);
		glVertex3f(-5.0f, -303.0f, 7.0f);
	glEnd();
	glDisable(GL_BLEND);
	glLineWidth(3.0f);
	glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
	glColor4f(0.7f, 0.7f, 0.0f, 1.0f);
	glBegin(GL_POLYGON);
		glVertex3f(-5.0f, -303.0f, 3.0f);
		glVertex3f(-5.0f, -297.0f, 3.0f);
		glVertex3f(-5.0f, -297.0f, 7.0f);
		glVertex3f(-5.0f, -303.0f, 7.0f);
	glEnd();
	glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
	glEnable(GL_LIGHTING);

	// draw text
	glTranslatef(-4.0f, -302.28f, 6.25f);
	glRotatef(90.0f, 0.0f, 0.0f, 1.0f);
	glRotatef(90.0f, 1.0f, 0.0f, 0.0f);
	glScalef(0.4f, 0.4f, 0.4f);
	GLfloat highdif[] = { 0.8f, 0.8f, 0.8f, 1.0f };
	GLfloat highamb[] = { 0.5f, 0.5f, 0.5f, 1.0f };
	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, highdif);
	glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, highamb);
	glPushMatrix();
	DrawStr("Halcyon Racer Best Times", m_dlBlueMaterial);
	glPopMatrix();
	DrawStr("_______________________", m_dlBlueMaterial);
	glTranslatef(-11.25f, -1.0f, 0.0f);
	
	for(int i = 0; i < 7; i++)
	{
		glPushMatrix();
			char buffer[10];
			sprintf(buffer, "%d ", i+1);
			DrawStr(buffer, m_dlBlueMaterial);
		glPopMatrix();
		glPushMatrix();
			glTranslatef(1.0f, 0.0f, 0.0f);
			if(i == m_nHighScorePlace)
			{	
				//crazy stuff to get it lined up correctly
				glTranslatef(m_nHighScoreInitialsPosition*0.5+(0.1f*(m_nHighScoreInitialsPosition+1)), 0.0f, 0.0f);	
				DrawStr("_", m_dlRedMaterial);	
				glTranslatef(-m_nHighScoreInitialsPosition*0.5f-0.45f-(0.1f*(m_nHighScoreInitialsPosition+1)),0.0f,0.0f);
				DrawStr(m_strHighScores[i][0]);
			}
			else
				DrawStr(m_strHighScores[i][0], m_dlBlueMaterial);
		glPopMatrix();
		glPushMatrix();
			glTranslatef(7.0f, 0.0f, 0.0f);
			DrawStr(m_strHighScores[i][1], m_dlBlueMaterial);
		glPopMatrix();
		glTranslatef(0.0f, -1.0f, 0.0f);
	}
}

//////////////////////////////////////////////////////////////////////////////////// keyboard handling

///////////////////////////////////////////////////////////////////
// CGameEngine::vWindowKeyboardHandler(CKeyState cKey)
// Description:	main method of getting key input and dealing with it for each gamestate
///////////////////////////////////////////////////////////////////
void CGameEngine::vWindowKeyboardHandler(CKeyState cKey)
{
	unsigned char chKey = cKey.GetKeyAsChar();

	switch(m_nGameState)
	{
	case GS_LOADINGSCREEN:
		break;

	case GS_ABOUTSCREEN:
		// hit any key, go back to mainscreen
		m_nNextGameState = GS_MAINSCREEN;
		m_nFadeType = FADE_OUT;
		m_fFade = 0.0f;
		break;

	case GS_MAINSCREEN:
		// move around in the menu
		switch (cKey.GetKeyAsChar())
		{
		case CKeyState::GLK_UP:
			m_cSound.StopBuffer(1);
			m_cSound.PlayBuffer(1, 0);
			m_nMenuItemSelected--;
			if(m_nMenuItemSelected < 0)
				m_nMenuItemSelected += 4;
			break;
		case CKeyState::GLK_DOWN:
			m_cSound.StopBuffer(1);
			m_cSound.PlayBuffer(1, 0);
			m_nMenuItemSelected++;
			if(m_nMenuItemSelected > 3)
				m_nMenuItemSelected -= 4;
			break;
		case CKeyState::GLK_ENTER:
			m_cSound.PlayBuffer(2, 0);
			switch(m_nMenuItemSelected)
			{
			case 0:
				m_fIntroStartTime = clock();
				m_nNextGameState = GS_GAMESTART;
				m_nFadeType = FADE_OUT;
				m_fFade = 0.0f;
				break;
			case 1:
				m_nNextGameState = GS_HIGHSCORE;
				m_nFadeType = FADE_OUT;
				m_fFade = 0.0f;
				break;
			case 2:
				m_nNextGameState = GS_ABOUTSCREEN;
				m_nFadeType = FADE_OUT;
				m_fFade = 0.0f;
				break;
			case 3:
				m_nNextGameState = GS_GAMECLOSE;
				m_nFadeType = FADE_OUT;
				m_fFade = 0.0f;
				break;
			default:
				break;
			}
			break;
		default:
			break;
		}
		break;

	case GS_HIGHSCORE:
		// hit any key, go back to mainscreen
		m_nNextGameState = GS_MAINSCREEN;
		m_nFadeType = FADE_OUT;
		m_fFade = 0.0f;
		break;

	case GS_GAMESTART:
		break;

	case GS_GAME:
		// hit escape, go back to mainscreen
		if (cKey.GetKeyAsChar() == CKeyState::GLK_ESCAPE)
		{
			m_nNextGameState = GS_MAINSCREEN;
			m_nFadeType = FADE_OUT;
			m_fFade = 0.0f;
			m_cEnv.m_cUserCar.ToggleInGame(false);
			m_cSound.StopBuffer(10);
		}
		break;

	case GS_GAMECLOSE:
		break;

	case GS_GAMEOVER:
		if (cKey.GetKeyAsChar() == CKeyState::GLK_ESCAPE)
		{
			m_nNextGameState = GS_MAINSCREEN;
			m_nFadeType = FADE_OUT;
			m_fFade = 0.0f;
		}
		break;

	case GS_NEWHIGHSCORE:
		// deal with entering a new high score
		switch (chKey)
		{
		case CKeyState::GLK_ESCAPE:
			m_nNextGameState = GS_MAINSCREEN;
			m_nFadeType = FADE_OUT;
			m_fFade = 0.0f;
			break;
		case CKeyState::GLK_ENTER:
			m_nNextGameState = GS_MAINSCREEN;
			m_nFadeType = FADE_OUT;
			m_fFade = 0.0f;
			break;
		case CKeyState::GLK_LEFT:
			if(m_nHighScoreInitialsPosition > 0)
				m_nHighScoreInitialsPosition--;
			else if(m_nHighScoreInitialsPosition == 0)
				m_nHighScoreInitialsPosition = 2;
			break;
		case CKeyState::GLK_RIGHT:
			if(m_nHighScoreInitialsPosition < 2)
				m_nHighScoreInitialsPosition++;
			else if(m_nHighScoreInitialsPosition == 2)
				m_nHighScoreInitialsPosition = 0;
			break;
		default:
			if(IsCharAlpha(chKey))
			{
				m_strHighScores[m_nHighScorePlace][0][m_nHighScoreInitialsPosition] = (char)CharUpper((LPTSTR)chKey);
				if(m_nHighScoreInitialsPosition < 2)
					m_nHighScoreInitialsPosition++;
				else if(m_nHighScoreInitialsPosition == 2)
					m_nHighScoreInitialsPosition = 0;
			}
			break;
		}
		break;
	default:
		break;
	}

	// 'q' is mapped as a quick way to quit right now
	if (chKey == 'q')
		PostQuitMessage(1);
}

///////////////////////////////////////////////////////////////////
// CGameEngine::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) 
// Description:	set flags for multiple key down
///////////////////////////////////////////////////////////////////
void CGameEngine::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	if(m_nGameState == GS_GAME)
	{
		if(nChar == VK_UP)
			m_bUp = true;
		else if(nChar == VK_DOWN)
			m_bDown = true;	
		else if(nChar == VK_LEFT)
			m_bLeft = true;		
		else if(nChar == VK_RIGHT)
			m_bRight = true;
		else
			m_chLastKeyHit = nChar;
	}
	COpenGLControlBase::OnKeyDown(nChar, nRepCnt, nFlags);
}

///////////////////////////////////////////////////////////////////
// CGameEngine::OnKeyUp(UINT nChar, UINT nRepCnt, UINT nFlags) 
// Description:	clear the flags for multiple key down
///////////////////////////////////////////////////////////////////
void CGameEngine::OnKeyUp(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	if(m_nGameState == GS_GAME)
	{
		if(nChar == VK_UP)
			m_bUp = false;
		if(nChar == VK_DOWN)
			m_bDown = false;	
		if(nChar == VK_LEFT)
			m_bLeft = false;		
		if(nChar == VK_RIGHT)
			m_bRight = false;		
	}
	COpenGLControlBase::OnKeyUp(nChar, nRepCnt, nFlags);
}

//////////////////////////////////////////////////////////////////////////////////// high score

///////////////////////////////////////////////////////////////////
// CGameEngine::CheckHighScore(string time)
// Description:	see if new high score should be added
///////////////////////////////////////////////////////////////////
int	CGameEngine::CheckHighScore(string time)
{
	int nReturnValue = 99;
	int i;
	for(i = 0; i < 7; i++)
	{
		if(time <= m_strHighScores[i][1])
		{
			nReturnValue = i;
			break;
		}
	}
	return nReturnValue;
}

///////////////////////////////////////////////////////////////////
// CGameEngine::LoadHighScores()
// Description:	load the high scores from file. if no file, make it
///////////////////////////////////////////////////////////////////
void CGameEngine::LoadHighScores()
{
	ifstream fIn("highscore.hlc");
	if(fIn)
	{
		for(int i = 0; i < 7; i++)
		{
			fIn >> m_strHighScores[i][0];
			fIn >> m_strHighScores[i][1];
		}
		fIn.close();
	}
	else
	{
		m_strHighScores[0][0] = "ELL";
		m_strHighScores[0][1] = "99:99.99";
		m_strHighScores[1][0] = "ELL";
		m_strHighScores[1][1] = "99:99.99";
		m_strHighScores[2][0] = "ELL";
		m_strHighScores[2][1] = "99:99.99";
		m_strHighScores[3][0] = "ELL";
		m_strHighScores[3][1] = "99:99.99";
		m_strHighScores[4][0] = "ELL";
		m_strHighScores[4][1] = "99:99.99";
		m_strHighScores[5][0] = "ELL";
		m_strHighScores[5][1] = "99:99.99";
		m_strHighScores[6][0] = "ELL";
		m_strHighScores[6][1] = "99:99.99";

		SaveHighScores();
	}
}

///////////////////////////////////////////////////////////////////
// CGameEngine::SaveHighScores()
// Description:	save out the new high scores
///////////////////////////////////////////////////////////////////
void CGameEngine::SaveHighScores()
{
	ofstream fOut("highscore.hlc");
	if(fOut)
	{
		for(int i = 0; i < 7; i++)
		{
			fOut << m_strHighScores[i][0] << " " << m_strHighScores[i][1] << endl;
		}
		fOut.close();
	}
	else
	{
		AfxMessageBox("Cannot write to High Score file");
	}

}