//		   Name: CGeneric.h
//
//	Description: Contains a generic base class from which all USC Irix
//				 classes should be derived.
//
//		Version		Date		Author		Comment	
//	   ----------- ----------- ----------- -----------
//		00.00.00	3/13/97		King		Creation
//		00.00.01	7/23/97		King		Modification of error reporting mechanism.
//											Added the error buffer.
//		00.02.00	8/7/97		King		Removal of definition of bool, true,
//											false, TRUE, and FALSE. MAX_STR_LEN
//											also removed in order to promote 
//											use of STL string class.
//		00.02.01	8/27/97		King		Use STL string (Modena version) for
//											the internal error string
//		00.02.01	11/20/97	King		Added nMaxStringSize to be used as a
//											global constant
//		00.02.02	11/21/97	King		Removed the filename manipulators and the
//											random number generator function.
//											Separated the classes CRandomGenerator and
//											CFilename
//		00.02.02	06/30/98	King		Removed #ifdef protection, replaced with
//											#pragma once
//		00.02.03	08/20/98	King		Added #ifdef _USC_DEBUG_MODE to automatically print
//											to stderr anytime an object set an error condition
//		00.03.00	10/28/98	King		Added first pass of exception handling service code,
//											removed all other error handlers
//		00.03.00	02/02/99	Weaver		Converted includes to std namespace versions.
//		00.03.00	07/30/99	King		Added note handling functions

#ifndef __sgi
#pragma warning (disable : 4786 4018 4275 4251)
#ifdef _BUILD_DLL
#define BUILD_GENERIC_SET __declspec(dllexport)
#else
#define BUILD_GENERIC_SET __declspec(dllimport)
#endif // _BUILD_DLL
#endif // __sgi

#pragma once


#ifdef _DEBUG
#pragma comment(lib, "Generic_v3.1d.lib")
#else
#pragma comment(lib, "Generic_v3.1.lib")
#endif  // _DEBUG


//
//	Include files
//


#ifdef __sgi
#include <stdlib.h>
#include <stdexcept.h>
#include <stdarg.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <errno.h>
#include <new.h>
#include <time.h>
#include <iostream.h>
#include <string>
#include <unistd.h>
#include <vector.h>
#define NS_SCOPE

#else // __sgi (Windows/NT Version)

#include <cstdlib>
#include <stdexcept>
#include <cstdarg>
#include <cstdio>
#include <string>
#include <cctype>
#include <cerrno>
#include <new.h>
#include <ctime>
#include <io.h>
#include <iostream>
#include <vector>
#define NS_SCOPE std::
#endif // __sgi

using namespace std;


//
//	These constants are placed here to avoid forcing the developer
//	to use the CGeneric class to access these global constants.
//

const int	Error = -1;
const int	Ok = 0;

const int	nMaxStringSize = 1024;
typedef char TmpString[nMaxStringSize];


//
//	System defined exception heirarchy
//
//	exception
//		logic_error
//			length_error
//			domain_error
//			out_of_range
//			invalid_argument
//		runtime_error
//			range_error
//			overflow_error
//			underflow_error
//		bad_alloc
//		bad_exception
//		ios_base::failure
//		bad_case
//		bad_typeid
//
//
//	USC/ECE defined exceptions:
//
// 		usc_parse_error			(runtime)
// 		usc_file_error			(runtime)
// 		usc_invalid_switch		(logic)
//


//		   Name: usc_user_error
//
//	Description:
//
// 		User errors are distinguished from logic errors in that the
//		"user" is defined to be the person actually using the computer,
//		not the person/programmer using a library.
//	
//
//		Version 	Date		Author		Comment	
//	   ----------- ----------- ----------- -----------
//		03.01.00	11/04/98	King		Creation

class BUILD_GENERIC_SET usc_user_error : public logic_error
{
public:
		usc_user_error(string strDetails) : 
			logic_error("User Error: [" + strDetails + "]")
		{
		}
};


//		   Name: usc_internal_error
//
//	Description:
//
// 		Logic errors are used to detect defects in coding.  Had the author
//		written the code perfectly, no logic exceptions would result.
//
//
//		Version 	Date		Author		Comment	
//	   ----------- ----------- ----------- -----------
//		03.01.00	11/04/98	King		Creation

class BUILD_GENERIC_SET usc_internal_error : public logic_error
{
public:
		usc_internal_error(string strDetails) :
			logic_error("Internal Error: [" + strDetails + "]")
		{
		}
};

typedef usc_internal_error usc_logic_error;



//		   Name: usc_system_error
//
//	Description:
//
// 		No matter how well written your code may be, runtime exceptions
//		may occur.  This is usually the result of an unexpected system
//		behvaior or bad user input.
//
//
//		Version 	Date		Author		Comment	
//	   ----------- ----------- ----------- -----------
//		03.01.00	11/04/98	King		Creation

class BUILD_GENERIC_SET usc_system_error : public runtime_error
{
public:
		usc_system_error(string strDetails, bool bAppendSystemError = false) : 
			runtime_error("System Error: [" + strDetails + (bAppendSystemError ? (" -- " + string(strerror(errno))) : "") + "]")
		{
		}
};



//		   Name: usc_file_error
//
//	Description:
//
// 		No matter how well written your code may be, runtime exceptions
//		may occur.  This is usually the result of an unexpected system
//		behvaior or bad user input.
//
//
//		Version 	Date		Author		Comment	
//	   ----------- ----------- ----------- -----------
//		03.01.00	11/04/98	King		Creation

class BUILD_GENERIC_SET usc_file_error : public runtime_error
{
public:
		usc_file_error(string strDetails, bool bAppendSystemError = false) : 
			runtime_error("File Error: [" + strDetails + (bAppendSystemError ? (" -- " + string(strerror(errno))) : "") + "]")
		{
		}
};


typedef usc_system_error usc_runtime_error;



//		   Name: CGeneric
//
//	Description: A generic base class from which all USC Irix classes should
//				 be derived.
//
//		Summary of public functions:
//
//			string	MakeString(const char *pField, ...);
//
//		Version		Date		Author		Comment	
//	   ----------- ----------- ----------- -----------
//		00.01.00	3/13/97		King		Creation
//		00.01.01	7/23/97		King		Modification of error reporting mechanism.  Added
//											the error buffer.
//		00.02.00	8/7/97		King		Removal of definition of bool, true, false, TRUE, and FALSE.
//											MAX_STR_LEN also removed in order to promote use of STL string class.
//		00.02.01	8/27/97		King		Use STL string (Modena version) for
//											the internal error string

class BUILD_GENERIC_SET CGeneric
{
public:
		CGeneric( )
		{
#ifdef __sgi
			set_new_handler(&CGeneric::NewErrorHandler);
#else
			_set_new_handler(&CGeneric::NewErrorHandler);
#endif
		}	

virtual ~CGeneric( )
		{
		}
		
static	void				AddNote(string strNote);
static	string				GetNote(int nIndex)						{ return m_vNotes[nIndex]; }							
static	int					GetNoteCount(void)						{ return m_vNotes.size( ); }
static	vector<string>		GetNotes(void);
static	string				MakeString(const char* pField, ...);

protected:

private:

#ifdef __sgi
static	void				NewErrorHandler(void);
#else
static	int					NewErrorHandler(size_t nSize);
#endif

static	vector<string>		m_vNotes;
};



//		   Name: CFilename
//
//	Description:
//
//		For a given filename "/home/doe/foo.cxx":
//			Get( )			-> "/home/doe/foo.cxx"
//			GetExtension( ) -> "cxx"				was GetExtension( )
//			GetPath( )		-> "/home/doe"			was GetPathname( )
//			NoExtension( )	-> "/home/doe/foo"		was BaseFilename( )
//			NoPath( )		-> "foo.cxx"			was LastFilename( )
//
//		Summary of defined methods:
//			string	Find(string strEnvVar)
//			string	Get(void)
//			string	GetExtension(void)
//			string	GetPath(void)
//			string	NoExtension(void)
//			string	NoPath(void)
//			void	Set(string strFilename);
//
//		Version 	Date		Author		Comment	
//	   ----------- ----------- ----------- -----------
//		02.01.00	11/21/97	King		Creation
//		03.01.00	06/30/98	King		Added lookup in environment variable paths

class BUILD_GENERIC_SET CFilename
{
public:
		CFilename( )
		{
		}
		CFilename(char* pFilename)
		{
			m_strFilename = string(pFilename);
		}
		CFilename(string strFilename)
		{
			m_strFilename = strFilename;
		}


		//		   Name: Find
		//
		//	Description:
		//
		//
		//		Version 	Date		Author		Comment	
		//	   ----------- ----------- ----------- -----------
		//		02.01.00	06/30/98	King		Creation

		string Find(string strEnvVar)
		{
			string strGoodFile = m_strFilename;
			char* pEnvValue = strdup(getenv(strEnvVar.c_str( )));
			if (pEnvValue)
			{
#ifdef __sgi
				if (access(m_strFilename.c_str( ), F_OK) == 0)
#else
				if (_access(m_strFilename.c_str( ), 0) == 0)
#endif
					strGoodFile = m_strFilename;
				else if (m_strFilename.length( ) > 0)
				{
					string strNoPath = (m_strFilename[0] == '/') ? CFilename(m_strFilename).NoPath( ) : m_strFilename;
					char* pPath = strtok((char*) pEnvValue, ":");
					while (pPath)
					{
						string strTestFile = string(pPath) + "/" + strNoPath;
#ifdef __sgi
						if (access(strTestFile.c_str( ), F_OK) == 0)
#else
						if (_access(strTestFile.c_str( ), 0) == 0)
#endif
						{
							strGoodFile = strTestFile;
							break;
						}
						pPath = strtok(NULL, ":");
					}
				}
			}
			else
			{
				//SetError("cannot find environment variable '%s'", strEnvVar.c_str( ));
			}
			if (pEnvValue)
				free(pEnvValue);

			return strGoodFile;
		}


		//		   Name: Get
		//
		//	Description:
		//
		//
		//		Version 	Date		Author		Comment	
		//	   ----------- ----------- ----------- -----------
		//		02.01.00	11/21/97	King		Creation

		string	Get(void)
		{
			return m_strFilename;
		}


		//		   Name: GetExtension
		//
		//	Description:
		//
		//
		//		Version 	Date		Author		Comment	
		//	   ----------- ----------- ----------- -----------
		//		02.01.00	11/21/97	King		Creation

		string GetExtension(void)
		{
			string strExtension;

			int nLength = m_strFilename.length( );
			for (int i = nLength - 1; i > 0 && m_strFilename[i] != '/'; i--)
				if (m_strFilename[i] == '.')
				{
					strExtension = string(m_strFilename.c_str( ) + i + 1);
					break;
				}

			return strExtension;
		}


		//		   Name: GetPath
		//
		//	Description:
		//
		//
		//		Version 	Date		Author		Comment	
		//	   ----------- ----------- ----------- -----------
		//		02.01.00	11/21/97	King		Creation

		string GetPath(void)
		{
			TmpString szPath;

			szPath[0] = '\0';
			int nLength = m_strFilename.length( );
			for (int i = nLength - 1; i >= 0; i--)
				if (m_strFilename[i] == '/' ||
					m_strFilename[i] == '\\')
				{
					for (int j = 0; j < i + 1; j++)
						szPath[j] = m_strFilename[j];
					szPath[j] = '\0';
					break;
				}

			return string(szPath);
		}


		//		   Name: NoExtension
		//
		//	Description:
		//
		//
		//		Version 	Date		Author		Comment	
		//	   ----------- ----------- ----------- -----------
		//		02.01.00	11/21/97	King		Creation

		string	NoExtension(void)
		{
			TmpString szExtension;

			strcpy(szExtension, m_strFilename.c_str( ));
			int nLength = m_strFilename.length( );
			for (int i = nLength - 1; i >= 0 && m_strFilename[i] != '/'; i--)
				if (m_strFilename[i] == '.')
				{
					for (int j = 0; j < i; j++)
						szExtension[j] = m_strFilename[j];
					szExtension[j] = '\0';
					break;
				}

			return string(szExtension);
		}


		//		   Name: NoPath
		//
		//	Description:
		//
		//
		//		Version 	Date		Author		Comment	
		//	   ----------- ----------- ----------- -----------
		//		02.01.00	11/21/97	King		Creation

		string	NoPath(void)
		{
			TmpString szPath;

			strcpy(szPath, m_strFilename.c_str( ));
			int nLength = m_strFilename.length( );
			for (int i = nLength - 1; i >= 0; i--)
				if (m_strFilename[i] == '/' || m_strFilename[i] == '\\')
				{
					for (int j = i + 1, k = 0; j < nLength; j++, k++)
						szPath[k] = m_strFilename[j];
					szPath[k] = '\0';
					break;
				}

			return string(szPath);
		}


		//		   Name: Set
		//
		//	Description:
		//
		//
		//		Version 	Date		Author		Comment	
		//	   ----------- ----------- ----------- -----------
		//		02.01.00	11/21/97	King		Creation

		void	Set(string strFilename)
		{
			m_strFilename = strFilename;
		}

private:
		string	m_strFilename;
};

